@@ -18,8 +18,8 @@
 
 use super::*;
 
-use std::num::FpCategory;
-use std::ops::*;
+use core::num::FpCategory;
+use core::ops::*;
 
 use num_traits::*;
 
@@ -31,7 +31,7 @@ macro_rules! impl_unary_ops {
             impl<T, N: ArrayLength<T>> $op_trait for NumericArray<T, N>
             where
                 T: $op_trait,
-                N: ArrayLength<<T as $op_trait>::Output>
+                N: ArrayLength<<T as $op_trait>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait>::Output, N>;
 
@@ -43,7 +43,7 @@ macro_rules! impl_unary_ops {
             impl<'a, T: Clone, N: ArrayLength<T>> $op_trait for &'a NumericArray<T, N>
             where
                 T: $op_trait,
-                N: ArrayLength<<T as $op_trait>::Output>
+                N: ArrayLength<<T as $op_trait>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait>::Output, N>;
 
@@ -61,7 +61,7 @@ macro_rules! impl_binary_ops {
             impl<T, U, N: ArrayLength<T> + ArrayLength<U>> $op_trait<NumericArray<U, N>> for NumericArray<T, N>
             where
                 T: $op_trait<U>,
-                N: ArrayLength<<T as $op_trait<U>>::Output>
+                N: ArrayLength<<T as $op_trait<U>>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait<U>>::Output, N>;
 
@@ -73,7 +73,7 @@ macro_rules! impl_binary_ops {
             impl<'a, T, U: Clone, N: ArrayLength<T> + ArrayLength<U>> $op_trait<&'a NumericArray<U, N>> for NumericArray<T, N>
             where
                 T: $op_trait<U>,
-                N: ArrayLength<<T as $op_trait<U>>::Output>
+                N: ArrayLength<<T as $op_trait<U>>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait<U>>::Output, N>;
 
@@ -85,7 +85,7 @@ macro_rules! impl_binary_ops {
             impl<'a, T: Clone, U, N: ArrayLength<T> + ArrayLength<U>> $op_trait<NumericArray<U, N>> for &'a NumericArray<T, N>
             where
                 T: $op_trait<U>,
-                N: ArrayLength<<T as $op_trait<U>>::Output>
+                N: ArrayLength<<T as $op_trait<U>>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait<U>>::Output, N>;
 
@@ -97,7 +97,7 @@ macro_rules! impl_binary_ops {
             impl<'a, 'b, T: Clone, U: Clone, N: ArrayLength<T> + ArrayLength<U>> $op_trait<&'b NumericArray<U, N>> for &'a NumericArray<T, N>
             where
                 T: $op_trait<U>,
-                N: ArrayLength<<T as $op_trait<U>>::Output>
+                N: ArrayLength<<T as $op_trait<U>>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait<U>>::Output, N>;
 
@@ -109,7 +109,7 @@ macro_rules! impl_binary_ops {
             impl<T, U: Clone, N: ArrayLength<T>> $op_trait<NumericConstant<U>> for NumericArray<T, N>
             where
                 T: $op_trait<U>,
-                N: ArrayLength<<T as $op_trait<U>>::Output>
+                N: ArrayLength<<T as $op_trait<U>>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait<U>>::Output, N>;
 
@@ -121,7 +121,7 @@ macro_rules! impl_binary_ops {
             impl<'a, T: Clone, U: Clone, N: ArrayLength<T>> $op_trait<NumericConstant<U>> for &'a NumericArray<T, N>
             where
                 T: $op_trait<U>,
-                N: ArrayLength<<T as $op_trait<U>>::Output>
+                N: ArrayLength<<T as $op_trait<U>>::Output>,
             {
                 type Output = NumericArray<<T as $op_trait<U>>::Output, N>;
 
@@ -129,6 +129,30 @@ macro_rules! impl_binary_ops {
                     NumericArray((&self.0).map(|l| $op_trait::$op(l.clone(), rhs.0.clone())))
                 }
             }
+
+            impl<T, U: Clone, N: ArrayLength<T>> $op_trait<NumericArray<T, N>> for NumericConstant<U>
+            where
+                U: $op_trait<T>,
+                N: ArrayLength<<U as $op_trait<T>>::Output>,
+            {
+                type Output = NumericArray<<U as $op_trait<T>>::Output, N>;
+
+                fn $op(self, rhs: NumericArray<T, N>) -> Self::Output {
+                    NumericArray(rhs.0.map(|r| $op_trait::$op(self.0.clone(), r)))
+                }
+            }
+
+            impl<'a, T: Clone, U: Clone, N: ArrayLength<T>> $op_trait<&'a NumericArray<T, N>> for NumericConstant<U>
+            where
+                U: $op_trait<T>,
+                N: ArrayLength<<U as $op_trait<T>>::Output>,
+            {
+                type Output = NumericArray<<U as $op_trait<T>>::Output, N>;
+
+                fn $op(self, rhs: &'a NumericArray<T, N>) -> Self::Output {
+                    NumericArray((&rhs.0).map(|r| $op_trait::$op(self.0.clone(), r.clone())))
+                }
+            }
         )*
     }
 }
@@ -138,7 +162,7 @@ macro_rules! impl_assign_ops {
         $(
             impl<T, U, N: ArrayLength<T> + ArrayLength<U>> $op_trait<NumericArray<U, N>> for NumericArray<T, N>
             where
-                T: $op_trait<U>
+                T: $op_trait<U>,
             {
                 fn $op(&mut self, rhs: NumericArray<U, N>) {
                     unsafe {
@@ -146,34 +170,34 @@ macro_rules! impl_assign_ops {
 
                         let (right_iter, right_position) = right.iter_position();
 
-                        for (lhs, rhs) in self.iter_mut().zip(right_iter) {
+                        self.iter_mut().zip(right_iter).for_each(|(lhs, rhs)| {
                             $op_trait::$op(lhs, ptr::read(rhs));
 
                             *right_position += 1;
-                        }
+                        });
                     }
                 }
             }
 
             impl<'a, T, U: Clone, N: ArrayLength<T> + ArrayLength<U>> $op_trait<&'a NumericArray<U, N>> for NumericArray<T, N>
             where
-                T: $op_trait<U>
+                T: $op_trait<U>,
             {
                 fn $op(&mut self, rhs: &'a NumericArray<U, N>) {
-                    for (lhs, rhs) in self.iter_mut().zip(rhs.iter()) {
+                    self.iter_mut().zip(rhs.iter()).for_each(|(lhs, rhs)| {
                         $op_trait::$op(lhs, rhs.clone());
-                    }
+                    });
                 }
             }
 
             impl<T, U: Clone, N: ArrayLength<T>> $op_trait<NumericConstant<U>> for NumericArray<T, N>
             where
-                T: $op_trait<U>
+                T: $op_trait<U>,
             {
                 fn $op(&mut self, rhs: NumericConstant<U>) {
-                    for lhs in self.iter_mut() {
+                    self.iter_mut().for_each(|lhs| {
                         $op_trait::$op(lhs, rhs.0.clone());
-                    }
+                    });
                 }
             }
         )*
@@ -185,7 +209,7 @@ macro_rules! impl_wrapping_ops {
         $(
             impl<T, N: ArrayLength<T>> num_traits::$op_trait for NumericArray<T, N>
             where
-                T: num_traits::$op_trait
+                T: num_traits::$op_trait,
             {
                 fn $op(&self, rhs: &Self) -> Self {
                     NumericArray((&self.0).zip(&rhs.0, num_traits::$op_trait::$op))
@@ -200,7 +224,7 @@ macro_rules! impl_checked_ops {
         $(
             impl<T, N: ArrayLength<T>> $op_trait for NumericArray<T, N>
             where
-                T: $op_trait
+                T: $op_trait,
             {
                 fn $op(&self, rhs: &Self) -> Option<Self> {
                     unsafe {
@@ -232,7 +256,7 @@ macro_rules! impl_float_const {
     ($($f:ident),*) => {
         impl<T, N: ArrayLength<T>> FloatConst for NumericArray<T, N>
         where
-            T: FloatConst
+            T: FloatConst,
         {
             $(
                 fn $f() -> Self {
@@ -406,7 +430,7 @@ where
     type FromStrRadixErr = <T as Num>::FromStrRadixErr;
 
     fn from_str_radix(str: &str, radix: u32) -> Result<Self, Self::FromStrRadixErr> {
-        <T as Num>::from_str_radix(str, radix).map(Self::from_element)
+        <T as Num>::from_str_radix(str, radix).map(Self::splat)
     }
 }
 
@@ -450,117 +474,41 @@ where
     }
 }
 
-impl<T, N: ArrayLength<T>> ToPrimitive for NumericArray<T, N>
-where
-    T: ToPrimitive,
-{
-    #[inline]
-    fn to_i64(&self) -> Option<i64> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_i64())
+macro_rules! impl_to_primitive {
+    ($($to:ident => $prim:ty),*) => {
+        impl<T, N: ArrayLength<T>> ToPrimitive for NumericArray<T, N>
+        where
+            T: ToPrimitive,
+        {
+            $(
+                #[inline]
+                fn $to(&self) -> Option<$prim> {
+                    if N::to_usize() == 0 { None } else {
+                        self.first().and_then(ToPrimitive::$to)
+                    }
+                }
+            )*
         }
     }
+}
 
-    #[inline]
-    fn to_u64(&self) -> Option<u64> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_u64())
-        }
-    }
+impl_to_primitive! {
+    to_i8       => i8,
+    to_i16      => i16,
+    to_i32      => i32,
+    to_i64      => i64,
+    to_i128     => i128,
+    to_isize    => isize,
 
-    #[inline]
-    fn to_isize(&self) -> Option<isize> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_isize())
-        }
-    }
+    to_u8       => u8,
+    to_u16      => u16,
+    to_u32      => u32,
+    to_u64      => u64,
+    to_u128     => u128,
+    to_usize    => usize,
 
-    #[inline]
-    fn to_i8(&self) -> Option<i8> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_i8())
-        }
-    }
-
-    #[inline]
-    fn to_i16(&self) -> Option<i16> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_i16())
-        }
-    }
-
-    #[inline]
-    fn to_i32(&self) -> Option<i32> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_i32())
-        }
-    }
-
-    #[inline]
-    fn to_usize(&self) -> Option<usize> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_usize())
-        }
-    }
-
-    #[inline]
-    fn to_u8(&self) -> Option<u8> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_u8())
-        }
-    }
-
-    #[inline]
-    fn to_u16(&self) -> Option<u16> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_u16())
-        }
-    }
-
-    #[inline]
-    fn to_u32(&self) -> Option<u32> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_u32())
-        }
-    }
-
-    #[inline]
-    fn to_f32(&self) -> Option<f32> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_f32())
-        }
-    }
-
-    #[inline]
-    fn to_f64(&self) -> Option<f64> {
-        if N::to_usize() == 0 {
-            None
-        } else {
-            self.first().and_then(|x| x.to_f64())
-        }
-    }
+    to_f32      => f32,
+    to_f64      => f64
 }
 
 impl<T, N: ArrayLength<T>> NumCast for NumericArray<T, N>
@@ -568,7 +516,7 @@ where
     T: NumCast + Clone,
 {
     fn from<P: ToPrimitive>(n: P) -> Option<Self> {
-        T::from(n).map(Self::from_element)
+        T::from(n).map(Self::splat)
     }
 }
 
@@ -579,37 +527,37 @@ where
 {
     #[inline]
     fn nan() -> Self {
-        Self::from_element(Float::nan())
+        Self::splat(Float::nan())
     }
 
     #[inline]
     fn infinity() -> Self {
-        Self::from_element(Float::infinity())
+        Self::splat(Float::infinity())
     }
 
     #[inline]
     fn neg_infinity() -> Self {
-        Self::from_element(Float::neg_infinity())
+        Self::splat(Float::neg_infinity())
     }
 
     #[inline]
     fn neg_zero() -> Self {
-        Self::from_element(Float::neg_zero())
+        Self::splat(Float::neg_zero())
     }
 
     #[inline]
     fn min_value() -> Self {
-        Self::from_element(Float::min_value())
+        Self::splat(Float::min_value())
     }
 
     #[inline]
     fn min_positive_value() -> Self {
-        Self::from_element(Float::min_positive_value())
+        Self::splat(Float::min_positive_value())
     }
 
     #[inline]
     fn max_value() -> Self {
-        Self::from_element(Float::max_value())
+        Self::splat(Float::max_value())
     }
 
     fn is_nan(self) -> bool {
@@ -703,19 +651,21 @@ where
             {
                 let (destination_iter, destination_position) = destination.iter_position();
 
-                for (dst, (l, (a, b))) in destination_iter.zip(left_iter.zip(a_arr_iter.zip(b_arr_iter))) {
-                    let l = ptr::read(l);
-                    let a = ptr::read(a);
-                    let b = ptr::read(b);
+                destination_iter
+                    .zip(left_iter.zip(a_arr_iter.zip(b_arr_iter)))
+                    .for_each(|(dst, (l, (a, b)))| {
+                        let l = ptr::read(l);
+                        let a = ptr::read(a);
+                        let b = ptr::read(b);
 
-                    *left_position += 1;
-                    *a_arr_position += 1;
-                    *b_arr_position += 1;
+                        *left_position += 1;
+                        *a_arr_position += 1;
+                        *b_arr_position += 1;
 
-                    ptr::write(dst, Float::mul_add(l, a, b));
+                        ptr::write(dst, Float::mul_add(l, a, b));
 
-                    *destination_position += 1;
-                }
+                        *destination_position += 1;
+                    });
             }
 
             NumericArray::new(destination.into_inner())
@@ -823,19 +773,22 @@ where
                 let (sin_destination_iter, sin_destination_position) = sin_destination.iter_position();
                 let (cos_destination_iter, cos_destination_position) = cos_destination.iter_position();
 
-                for ((sin, cos), src) in sin_destination_iter.zip(cos_destination_iter).zip(source_iter) {
-                    let x = ptr::read(src);
+                sin_destination_iter
+                    .zip(cos_destination_iter)
+                    .zip(source_iter)
+                    .for_each(|((sin, cos), src)| {
+                        let x = ptr::read(src);
 
-                    *source_position += 1;
+                        *source_position += 1;
 
-                    let (s, c) = Float::sin_cos(x);
+                        let (s, c) = Float::sin_cos(x);
 
-                    ptr::write(sin, s);
-                    ptr::write(cos, c);
+                        ptr::write(sin, s);
+                        ptr::write(cos, c);
 
-                    *sin_destination_position += 1;
-                    *cos_destination_position += 1;
-                }
+                        *sin_destination_position += 1;
+                        *cos_destination_position += 1;
+                    });
             }
 
             (
@@ -887,7 +840,7 @@ where
 
     #[inline]
     fn epsilon() -> Self {
-        Self::from_element(Float::epsilon())
+        Self::splat(Float::epsilon())
     }
 
     fn to_degrees(self) -> Self {
